package dev.fitko.fitconnect.api.config;

import lombok.Getter;

import java.net.URI;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Getter
public enum SchemaConfig {

    SCHEMA_BASE_URL(URI.create("https://schema.fitko.de/fit-connect/")),
    EVENTS_SCHEMA_PATH(SCHEMA_BASE_URL.schemaUri.resolve("events/")),
    SET_V_1_0_1(SCHEMA_BASE_URL.schemaUri.resolve("set-payload/1.0.1/set-payload.schema.json"), "set_schema_1.0.1.json"),
    SET_V_1_0_0(SCHEMA_BASE_URL.schemaUri.resolve("set-payload/1.0.0/set-payload.schema.json"), "set_schema_1.0.0.json"),
    METADATA_V_1_0_0(SCHEMA_BASE_URL.schemaUri.resolve("metadata/1.0.0/metadata.schema.json"), "metadata_schema_1.0.0.json"),

    XZUFI_DESTINATION_SCHEMA(SCHEMA_BASE_URL.schemaUri.resolve("xzufi/destination.schema.json"), "destination_schema.json");

    private final URI schemaUri;

    private final String fileName;

    SchemaConfig(final URI schemaUri, final String fileName) {
        this.schemaUri = schemaUri;
        this.fileName = fileName;
    }

    SchemaConfig(final URI schemaUri) {
        this.schemaUri = schemaUri;
        fileName = "";
    }

    public static List<String> getSetSchemaFilePaths(final String setSchemaBaseDir) {
        return Stream.of(SET_V_1_0_0.fileName, SET_V_1_0_1.fileName)
                .map(fileName -> setSchemaBaseDir + "/" + fileName)
                .collect(Collectors.toList());
    }

    public static List<String> getMetadataSchemaFileNames(final String metadataBaseDir) {
        return Stream.of(METADATA_V_1_0_0.fileName)
                .map(fileName -> metadataBaseDir + "/" + fileName)
                .collect(Collectors.toList());
    }

    public static List<String> getDestinationSchemaPaths(final String destinationSchemaBaseDir) {
        return Stream.of(XZUFI_DESTINATION_SCHEMA.fileName)
                .map(fileName -> destinationSchemaBaseDir + "/" + fileName)
                .collect(Collectors.toList());
    }

    @Override
    public String toString() {
        return schemaUri.toString();
    }

}
