package dev.fitko.fitconnect.api.domain.model.event;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import static dev.fitko.fitconnect.api.config.SchemaConfig.EVENTS_SCHEMA_PATH;

public enum Event {

    CREATE("create-submission") {
        @Override
        public boolean hasAuthTags() {
            return false;
        }

        public SubmissionState getState() {
            return SubmissionState.INCOMPLETE;
        }

        @Override
        public Set<Event> allowedNextEvents() {
            return Set.of(SUBMIT);
        }
    },

    SUBMIT("submit-submission") {
        @Override
        public boolean hasAuthTags() {
            return true;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.SUBMITTED;
        }

        @Override
        public Set<Event> allowedNextEvents() {
            return Set.of(ACCEPT, REJECT, FORWARD, NOTIFY);
        }
    },

    FORWARD("forward-submission") {
        @Override
        public boolean hasAuthTags() {
            return false;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.FORWARDED;
        }

        @Override
        public Set<Event> allowedNextEvents() {
            return Set.of(ACCEPT, REJECT, DELETE);
        }
    },

    ACCEPT("accept-submission") {
        @Override
        public boolean hasAuthTags() {
            return true;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.ACCEPTED;
        }

        @Override
        public Set<Event> allowedNextEvents() {
            return Set.of(DELETE);
        }
    },

    REJECT("reject-submission") {
        @Override
        public boolean hasAuthTags() {
            return false;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.REJECTED;
        }

        @Override
        public Set<Event> allowedNextEvents() {
            return Set.of(DELETE);
        }
    },

    NOTIFY("notify-submission") {
        @Override
        public boolean hasAuthTags() {
            return false;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.SUBMITTED;
        }

        @Override
        public Set<Event> allowedNextEvents() {
            return Set.of(SUBMIT);
        }
    },

    DELETE("delete-submission") {
        @Override
        public boolean hasAuthTags() {
            return false;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.DELETED;
        }

        @Override
        public Set<Event> allowedNextEvents() {
            return Set.of(this);
        }
    };

    private static final Map<String, Event> CONSTANTS = new HashMap<>();

    static {
        for (final Event c : values()) {
            CONSTANTS.put(c.getSchemaUri(), c);
        }
    }

    private final String name;

    Event(final String name) {
        this.name = name;
    }

    public static Event fromSchemaUri(final String schemaUri) {
        return CONSTANTS.get(schemaUri);
    }

    public String getSchemaUri() {
        return EVENTS_SCHEMA_PATH.getSchemaUri() + name;
    }

    public abstract boolean hasAuthTags();

    public abstract SubmissionState getState();

    public abstract Set<Event> allowedNextEvents();
}
