package dev.fitko.fitconnect.api.domain.model.event;

import dev.fitko.fitconnect.api.domain.model.event.problems.Problem;
import dev.fitko.fitconnect.api.domain.model.submission.Submission;
import lombok.Data;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.UUID;

@Data
public class EventPayload {

    private UUID submissionId;
    private UUID destinationId;
    private UUID caseId;
    private String encryptedMetadata;
    private String encryptedData;
    private Map<UUID, String> encryptedAttachments;
    private List<Problem> problems;

    public EventPayload(final Submission submission, final Map<UUID, String> encryptedAttachments, final List<Problem> problems) {
        submissionId = submission.getSubmissionId();
        destinationId = submission.getDestinationId();
        caseId = submission.getCaseId();
        encryptedData = submission.getEncryptedData();
        encryptedMetadata = submission.getEncryptedMetadata();
        this.encryptedAttachments = encryptedAttachments;
        this.problems = problems;
    }

    public EventPayload(final Submission submission, final Map<UUID, String> encryptedAttachments) {
        this(submission, encryptedAttachments, Collections.emptyList());
    }

    public EventPayload(final Submission submission, final List<Problem> problems) {
        this(submission, Collections.emptyMap(), problems);
    }

    public EventPayload(final Submission submission) {
        this(submission, Collections.emptyMap(), Collections.emptyList());
    }
}
