package dev.fitko.fitconnect.api.domain.validation;

/**
 * Wrapper for validations including an exception
 */
public final class ValidationResult {

    private final boolean isValid;
    private Exception error;

    private ValidationResult(final boolean isValid) {
        this.isValid = isValid;
    }

    private ValidationResult(final boolean isValid, final Exception exception) {
        this.isValid = isValid;
        this.error = exception;
    }

    public static ValidationResult ok() {
        return new ValidationResult(true);
    }

    public static ValidationResult error(final Exception exception) {
        return new ValidationResult(false, exception);
    }

    public boolean isValid() {
        return this.isValid;
    }

    public boolean hasError() {
        return !this.isValid || this.error != null;
    }

    public Exception getError() {
        return this.error;
    }
}
