package dev.fitko.fitconnect.client.cli;

import com.beust.jcommander.JCommander;
import com.beust.jcommander.ParameterException;
import dev.fitko.fitconnect.client.SenderClient;
import dev.fitko.fitconnect.client.SubscriberClient;
import dev.fitko.fitconnect.client.cli.batch.CsvImporter;
import dev.fitko.fitconnect.client.cli.commands.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;

class CommandLineClient {

    private static final Logger LOGGER = LoggerFactory.getLogger(CommandLineClient.class);

    private final SendSubmissionCommand sendSubmissionCommand;
    private final ListAllSubmissionsCommand listAllSubmissionsCommand;
    private final GetOneSubmissionCommand getOneSubmissionCommand;
    private final GetAllSubmissionsCommand getAllSubmissionsCommand;
    private final SendBatchCommand sendBatchCommand;

    private final CommandExecutor commandExecutor;

    private final JCommander jc;

    CommandLineClient(final SenderClient senderClient, final SubscriberClient subscriberClient) {

        this.sendSubmissionCommand = new SendSubmissionCommand();
        this.listAllSubmissionsCommand = new ListAllSubmissionsCommand();
        this.getOneSubmissionCommand = new GetOneSubmissionCommand();
        this.getAllSubmissionsCommand = new GetAllSubmissionsCommand();
        this.sendBatchCommand = new SendBatchCommand();

        this.commandExecutor = new CommandExecutor(senderClient, subscriberClient, new CsvImporter());

        this.jc = getJCommander();
    }

    void run(final String... args) {
        final var parsedCommand = parseCommandFromArguments(args);
        if (parsedCommand == null) {
            this.jc.usage();
            return;
        }
        try {
            executeCommand(parsedCommand);
        } catch (final Exception e) {
            LOGGER.error(e.getMessage(), e);
        }
    }

    private JCommander getJCommander() {
        return JCommander.newBuilder()
                .addCommand(this.sendSubmissionCommand)
                .addCommand(this.listAllSubmissionsCommand)
                .addCommand(this.getOneSubmissionCommand)
                .addCommand(this.getAllSubmissionsCommand)
                .addCommand(this.sendBatchCommand)
                .build();
    }

    private void executeCommand(final String parsedCommand) throws IOException {
        switch (parsedCommand) {
            case SendSubmissionCommand.SEND_CMD_NAME:
                this.commandExecutor.sendSubmission(this.sendSubmissionCommand);
                break;
            case ListAllSubmissionsCommand.LIST_CMD_NAME:
                this.commandExecutor.listSubmissions(this.listAllSubmissionsCommand);
                break;
            case GetOneSubmissionCommand.GET_CMD_NAME:
                this.commandExecutor.getOneSubmission(this.getOneSubmissionCommand);
                break;
            case GetAllSubmissionsCommand.GET_ALL_CMD_NAME:
                this.commandExecutor.getAllSubmissions(this.getAllSubmissionsCommand);
                break;
            case SendBatchCommand.SEND_BATCH_CMD_NAME:
                this.commandExecutor.sendBatch(this.sendBatchCommand);
                break;
            default:
                break;
        }
    }

    private String parseCommandFromArguments(final String[] args) {
        try {
            this.jc.parse(args);
            return this.jc.getParsedCommand();
        } catch (final ParameterException e) {
            LOGGER.error(e.getMessage(), "\n");
            return null;
        }
    }
}
