package dev.fitko.fitconnect.client.cli.batch;

import com.opencsv.bean.CsvToBeanBuilder;
import com.opencsv.bean.exceptionhandler.CsvExceptionHandler;
import dev.fitko.fitconnect.api.exceptions.BatchImportException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.Reader;
import java.util.List;

public class CsvImporter implements BatchImporter {

    private static final Logger LOGGER = LoggerFactory.getLogger(CsvImporter.class);

    public static final char CSV_DELIMITER = ',';

    @Override
    public List<ImportRecord> readRecords(final String path) {
        try {
            return readToImportRecords(getReader(path));
        } catch (final Exception e) {
            throw new BatchImportException(e.getMessage(), e);
        }
    }

    private BufferedReader getReader(final String path) throws FileNotFoundException {
        return new BufferedReader(new FileReader(path));
    }

    private List<ImportRecord> readToImportRecords(final Reader reader) {
        return new CsvToBeanBuilder<ImportRecord>(reader)
                .withType(ImportRecord.class)
                .withSeparator(CSV_DELIMITER)
                .withIgnoreLeadingWhiteSpace(true)
                .withIgnoreEmptyLine(true)
                .withThrowExceptions(true)
                .withExceptionHandler(getCsvExceptionHandler())
                .build()
                .parse();
    }

    private CsvExceptionHandler getCsvExceptionHandler() {
        return e -> {
            LOGGER.error("Could not read csv line {} with content \n {}", e.getLineNumber(), e.getLine(), e);
            return e;
        };
    }
}
