package dev.fitko.fitconnect.client.cli.commands;

import com.beust.jcommander.Parameter;
import com.beust.jcommander.Parameters;
import dev.fitko.fitconnect.client.cli.util.AttachmentDataType;
import dev.fitko.fitconnect.client.cli.util.UUIDConverter;
import dev.fitko.fitconnect.client.cli.util.UUIDValidator;
import lombok.Data;

import java.util.Collections;
import java.util.List;
import java.util.UUID;


@Data
@Parameters(
        commandNames = {SendSubmissionCommand.SEND_CMD_NAME},
        commandDescription = "Send a submission",
        separators = "="
)
public class SendSubmissionCommand {

    public static final String SEND_CMD_NAME = "send";

    @Parameter(names = {"--destinationId"}, converter = UUIDConverter.class, description = "Unique destination identifier in UUID format", validateWith = UUIDValidator.class, arity = 1, required = true)
    public
    UUID destinationId;

    @Parameter(names = {"--serviceName"}, arity = 1, description = "Name of the service type", required = true)
    public
    String serviceName;

    @Parameter(names = {"--leikaKey"}, arity = 1, description = "The LeikaKey of the service type", required = true)
    public
    String leikaKey;

    @Parameter(names = {"--data"}, arity = 1, description = "Path to JSON or XML data", required = true)
    public
    String data;

    @Parameter(names = {"--dataType"}, arity = 1, description = "Data mime type (json/xml), default JSON", required = true)
    public
    AttachmentDataType dataType;

    @Parameter(names = "--attachments", description = "Attachments as list of paths", variableArity = true)
    public
    List<String> attachments = Collections.emptyList();

}

