package dev.fitko.fitconnect.client.sender;

import dev.fitko.fitconnect.api.domain.model.submission.ServiceType;
import dev.fitko.fitconnect.client.sender.model.EncryptedSubmissionPayload;
import dev.fitko.fitconnect.client.sender.steps.EncryptedBuildStep;
import dev.fitko.fitconnect.client.sender.steps.EncryptedBuilderStartStep;
import dev.fitko.fitconnect.client.sender.steps.EncryptedDataStep;
import dev.fitko.fitconnect.client.sender.steps.EncryptedDestinationStep;
import dev.fitko.fitconnect.client.sender.steps.EncryptedMetadataStep;
import dev.fitko.fitconnect.client.sender.steps.EncryptedServiceTypeStep;
import lombok.Getter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collections;
import java.util.Map;
import java.util.UUID;

@Getter
public final class EncryptedSubmissionBuilder implements EncryptedBuilderStartStep,
        EncryptedDataStep,
        EncryptedMetadataStep,
        EncryptedDestinationStep,
        EncryptedServiceTypeStep,
        EncryptedBuildStep {

    private static final Logger LOGGER = LoggerFactory.getLogger(EncryptedSubmissionBuilder.class);

    private UUID destinationId;
    private Map<UUID, String> encryptedAttachments = Collections.emptyMap();
    private ServiceType serviceType;
    private String encryptedData;
    private String encryptedMetadata;

    private EncryptedSubmissionBuilder() {
    }

    public static EncryptedBuilderStartStep Builder() {
        return new EncryptedSubmissionBuilder();
    }

    @Override
    public EncryptedServiceTypeStep withDestination(final UUID destinationId) {
        this.destinationId = destinationId;
        return this;
    }

    @Override
    public EncryptedDataStep withEncryptedAttachments(final Map<UUID, String> encryptedAttachments) {
        this.encryptedAttachments = encryptedAttachments == null ? Collections.emptyMap() : encryptedAttachments;
        return this;
    }

    @Override
    public EncryptedDataStep withEncryptedAttachment(final UUID attachmentId, final String encryptedAttachment) {
        if (attachmentId == null || encryptedAttachment == null) {
            return withEncryptedAttachments(Collections.emptyMap());
        }
        return withEncryptedAttachments(Map.of(attachmentId, encryptedAttachment));
    }

    @Override
    public EncryptedMetadataStep withEncryptedData(final String encryptedData) {
        this.encryptedData = encryptedData;
        return this;
    }

    @Override
    public EncryptedDestinationStep withEncryptedMetadata(final String encryptedMetadata) {
        this.encryptedMetadata = encryptedMetadata;
        return this;
    }

    @Override
    public EncryptedBuildStep withServiceType(final String serviceTypeName, final String leikaKey) {
        serviceType = ServiceType.builder()
                .identifier(leikaKey)
                .name(serviceTypeName)
                .build();
        return this;
    }

    @Override
    public EncryptedBuildStep withServiceType(final String serviceTypeName, final String description, final String leikaKey) {
        serviceType = ServiceType.builder()
                .identifier(leikaKey)
                .description(description)
                .name(serviceTypeName)
                .build();
        return this;
    }

    @Override
    public EncryptedSubmissionPayload build() {
        return new EncryptedSubmissionPayload(this);
    }
}
