package dev.fitko.fitconnect.client.sender.steps;

import java.io.File;
import java.util.List;

public interface BuilderStartStep {

    /**
     * Sends the submission with a list of attachments
     *
     * @param attachments that are sent with the submission
     * @return the next step where additional data can be added to the submission
     */
    DataStep withAttachments(List<File> attachments);


    /**
     * Sends the submission with an attachments
     *
     * @param attachment that is sent with the submission
     * @return the next step where additional data can be added to the submission
     */
    DataStep withAttachment(File attachment);


    /**
     * JSON data as string.
     *
     * @param data json string
     * @return next step to submit the data
     */
    DestinationStep withJsonData(String data);

    /**
     * XML data as string.
     *
     * @param data xml string
     * @return next step to submit the data
     */
    DestinationStep withXmlData(String data);
}
