package dev.fitko.fitconnect.client.sender.steps;

import java.util.Map;
import java.util.UUID;

public interface EncryptedBuilderStartStep {

    /**
     * Sends the submission with a list of already encrypted attachments
     *
     * @param encryptedAttachments map of encrypted attachments on attachment UUIDs
     *
     * @return the next step where additional encrypted data can be added to the submission
     */
    EncryptedDataStep withEncryptedAttachments(Map<UUID, String> encryptedAttachments);

    /**
     * Sends the submission with an attachments
     *
     * @param attachmentId unique identifier attachment that is uploaded
     * @param attachment encrypted attachment that is sent with the submission
     * @return the next step where additional data can be added to the submission
     */
    EncryptedDataStep withEncryptedAttachment(UUID attachmentId, String attachment);

    /**
     * Data as encrypted JWE string.
     *
     * @param encryptedData encrypted data as string
     * @return next step to submit the data
     */
    EncryptedMetadataStep withEncryptedData(String encryptedData);
}
