package dev.fitko.fitconnect.client.util;

import com.nimbusds.jose.JWEObject;
import dev.fitko.fitconnect.api.domain.model.event.authtags.AuthenticationTags;
import dev.fitko.fitconnect.api.domain.model.submission.CreateSubmission;
import dev.fitko.fitconnect.api.domain.model.submission.SentSubmission;
import dev.fitko.fitconnect.api.domain.model.submission.ServiceType;
import dev.fitko.fitconnect.api.domain.model.submission.Submission;
import dev.fitko.fitconnect.api.domain.model.submission.SubmitSubmission;
import dev.fitko.fitconnect.client.sender.model.AttachmentPayload;

import java.text.ParseException;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * Helper to create types for announced, sent and submit submissions.
 */
public final class SubmissionUtil {

    private SubmissionUtil() {
    }

    public static CreateSubmission buildCreateSubmission(final UUID destinationId, final List<AttachmentPayload> attachments, final ServiceType serviceType) {
        return CreateSubmission.builder()
                .destinationId(destinationId)
                .announcedAttachments(attachments.stream().map(AttachmentPayload::getAttachmentId).collect(Collectors.toList()))
                .serviceType(serviceType)
                .build();
    }

    public static SubmitSubmission buildSubmitSubmission(final UUID submissionId, final String encryptedData, final String encryptedMetadata) {
        final SubmitSubmission submission = new SubmitSubmission();
        submission.setSubmissionId(submissionId);
        submission.setEncryptedData(encryptedData);
        submission.setEncryptedMetadata(encryptedMetadata);
        return submission;
    }

    public static SentSubmission buildSentSubmission(final List<AttachmentPayload> encryptedAttachments, final String encryptedData, final String encryptedMetadata, final Submission submission) {

        final AuthenticationTags authenticationTags = new AuthenticationTags();
        authenticationTags.setData(getAuthTag(encryptedData));
        authenticationTags.setMetadata(getAuthTag(encryptedMetadata));
        authenticationTags.setAttachments(getAuthTagsFromAttachments(encryptedAttachments));

        return SentSubmission.builder()
                .submissionId(submission.getSubmissionId())
                .caseId(submission.getCaseId())
                .destinationId(submission.getDestinationId())
                .authenticationTags(authenticationTags)
                .build();
    }

    private static Map<UUID, String> getAuthTagsFromAttachments(final List<AttachmentPayload> encryptedAttachments) {
        return encryptedAttachments.stream().collect(Collectors.toMap(AttachmentPayload::getAttachmentId, SubmissionUtil::getAuthTag));
    }

    private static String getAuthTag(final AttachmentPayload attachmentPayload) {
        return getAuthTag(attachmentPayload.getEncryptedData());
    }

    private static String getAuthTag(final String encryptedJWTData) {
        try {
            return JWEObject.parse(encryptedJWTData).getAuthTag().toString();
        } catch (final ParseException | NullPointerException e) {
            return "";
        }
    }
}
