package dev.fitko.fitconnect.core.util;

import dev.fitko.fitconnect.api.exceptions.FileHandlingException;
import dev.fitko.fitconnect.api.exceptions.RootCertificateException;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

public class FileUtil {

    public static List<String> loadContentOfFilesInDirectory(String directoryPath) {

        File folder = new File(Objects.requireNonNull(FileUtil.class.getClassLoader().getResource(directoryPath)).getFile());

        return Arrays.stream(Objects.requireNonNull(folder.listFiles())).map(file -> {
            try {
                return Files.readString(file.toPath());
            } catch (IOException e) {
                throw new FileHandlingException(e);
            }
        }).collect(Collectors.toList());
    }

    public static String loadContentOfFile(String filePath) {

        try {
            return Files.readString(new File(Objects.requireNonNull(FileUtil.class.getClassLoader().getResource(filePath)).getFile()).toPath());
        } catch (IOException e) {
            throw new FileHandlingException(e);
        }
    }

    public static X509Certificate convertToX509Certificate(String certificateString) {

        InputStream certificateInputStream = new ByteArrayInputStream(certificateString.getBytes());
        try {
            return (X509Certificate) CertificateFactory.getInstance("X.509").generateCertificate(certificateInputStream);
        } catch (CertificateException e) {
            throw new RootCertificateException(e);
        }
    }
}
