package dev.fitko.fitconnect.core.http;

import com.github.tomakehurst.wiremock.WireMockServer;
import com.github.tomakehurst.wiremock.client.WireMock;
import dev.fitko.fitconnect.api.config.BuildInfo;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.http.HttpMethod;
import org.springframework.web.client.RestTemplate;

import static com.github.tomakehurst.wiremock.client.WireMock.*;

public class ProxyRestTemplateTest {

    private final WireMockServer wireMockServer = new WireMockServer();

    @BeforeEach
    public void startWireMock() {
        wireMockServer.start();
    }

    @AfterEach
    public void shutDownWireMock() {
        wireMockServer.stop();
    }

    @Test
    void userAgentIsCorrect() {

        WireMock.configureFor("localhost", 8080);
        wireMockServer.stubFor(get(urlEqualTo("/test"))
                .withHeader("User-Agent", equalTo("productName/productVersion (commit:commit;os:" + System.getProperty("os.name") + ")"))
                .willReturn(ok().withBody(""))
        );

        final BuildInfo buildInfo = new BuildInfo();
        buildInfo.setProductName("productName");
        buildInfo.setProductVersion("productVersion");
        buildInfo.setCommit("commit");
        final RestTemplate restTemplate = new RestService(null, 0, buildInfo).getRestTemplate();

        restTemplate.exchange("http://localhost:8080/test", HttpMethod.GET, null, String.class);

        verify(getRequestedFor(urlPathEqualTo("/test"))
                .withHeader("User-Agent", equalTo("productName/productVersion (commit:commit;os:" + System.getProperty("os.name") + ")")));
    }
}
