package dev.fitko.fitconnect.core.http;

import dev.fitko.fitconnect.api.config.BuildInfo;
import org.junit.jupiter.api.Test;
import org.springframework.http.client.InterceptingClientHttpRequestFactory;
import org.springframework.web.client.RestTemplate;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.equalTo;
import static org.junit.jupiter.api.Assertions.*;

class RestServiceTest {

    RestService underTest;

    @Test
    void proxyRestTemplate() {

        // Given
        underTest = new RestService("http://testhost.de", 8080, new BuildInfo());

        // When
        final RestTemplate template = underTest.getRestTemplate();

        // Then
        assertNotNull(template);
        assertThat(template.getRequestFactory().getClass(), equalTo(InterceptingClientHttpRequestFactory.class));

    }

    @Test
    void hasProxySet() {

        // Given
        underTest = new RestService("http://testhost.de", 8080, new BuildInfo());

        // When
        final boolean proxySet = underTest.hasProxySet();

        // Then
        assertTrue(proxySet);
    }

    @Test
    void hasProxyNotSet() {

        // Given
        underTest = new RestService(null, 0, new BuildInfo());

        // When
        final boolean proxySet = underTest.hasProxySet();

        // Then
        assertFalse(proxySet);
    }

    @Test
    void testToString() {

        // Given
        underTest = new RestService("http://testhost.de", 8080, new BuildInfo());

        // When
        final String configAsString = underTest.toString();

        // Then
        assertEquals("ProxyConfig {host='http://testhost.de', port=8080}", configAsString);
    }
}